/* asm9830: a portable cross-assembler for the HP 9830A/B calculator */
/* Copyright (c) 2013-2014, Robert Ferguson (rob@bitscience.ca) */
/* All rights reserved; see LICENSE for details */

struct _expr {
	YYLTYPE loc;
	int	type;
	int	value;
	Symbol	*s;
	struct _expr *left;
	struct _expr *right;
	struct _expr *next; /* for expr lists */
};

extern Expr *numexpr(YYLTYPE loc, int value);
extern Expr *symbolexpr(YYLTYPE loc, Symbol *s);
extern Expr *opexpr(YYLTYPE loc, int op, Expr *l, Expr *r);
extern int evalexpr(Expr *e, int page, int *error, YYLTYPE *eloc);
extern char *errexpr(int err);

#define EXPR_VALUE		0x1
#define EXPR_SYMBOL		0x2
#define EXPR_OP			0x3

/* errors independent so we can accumulate them over a series of calls to evalexp() */

#define EXPR_SYMBOL_FORWARD	0x4
#define EXPR_ERROR		0x8
#define EXPR_NULL		0x10


#define OP_PLUS 	0x1
#define OP_MINUS	0x2
#define OP_MUL		0x3
#define OP_DIV		0x4
#define OP_AND		0x5
#define OP_OR		0x6
#define OP_XOR		0x7
#define OP_SHL		0x8
#define OP_SHR		0x9
#define OP_NEG		0xa
#define OP_TWDL		0xb

/* convenience macros for doing page-relative address ops */

#define PAGEINDEX(x)	((x) >> 11)
#define PAGEOFFSET(x)	((x) & ((1<<11) - 1))

