	/* asm9830: a portable cross-assembler for the HP 9830A/B calculator */
	/* Copyright (c) 2013-2014, Robert Ferguson (rob@bitscience.ca) */
	/* All rights reserved; see LICENSE for details */

%{
	#include <stdio.h>
	#include <stdlib.h>
	#include <string.h>
	#include <ctype.h>
	
	#include "parser.tab.h"
	#include "asm9830.h"

	int yycolumn = 1;

	extern char *yyfilename;
	extern unsigned long yyoffset;
	extern unsigned long yylineoffset;

#define YY_USER_ACTION yylloc->filename = yyfilename; \
	yylloc->first_line = yylloc->last_line = yylineno; \
	yylloc->first_column = yycolumn; yylloc->last_column = yycolumn+yyleng-1; \
	yylloc->lineoffset = yylineoffset; \
	yycolumn += yyleng; \
	yyoffset += yyleng; 
%}

%option noyywrap
%option nounput
%option noinput
%option yylineno
%option bison-bridge bison-locations

%x IN_COMMENT

DIGIT	 [0-9]
HEXDIGIT [0-9a-f]
OCTALDIGIT [0-7]
ID	((\.)[\._a-z0-9]+)|([_a-z][\._a-z0-9]*)
REG	[ab]
MOD	[ISC]

%%
	/* integers */

0x{HEXDIGIT}+ {
	yylval->num = (double)strtol(yytext, NULL, 16);
	return(NUM);
}

0{OCTALDIGIT}+ {
	yylval->num = (double)strtol(yytext, NULL, 8);
	return(NUM);
}

{DIGIT}+ {
	yylval->num = (double)strtol(yytext, NULL, 10);
	return(NUM);
}

	/* single character constant */

'[[:print:]]' {
	yylval->num = yytext[1];
	return(NUM);
}

	/* opcodes */

AD{REG} {
	yylval->opreg = tolower(yytext[2]) - 'a';
	return(T_ADR);
}

CP{REG} {
	yylval->opreg = tolower(yytext[2]) - 'a';
	return(T_CPR);
}

LD{REG} {
	yylval->opreg = tolower(yytext[2]) - 'a';
	return(T_LDR);
}

ST{REG} {
	yylval->opreg = tolower(yytext[2]) - 'a';
	return(T_STR);
}

IOR {
	return(T_IOR);
}

ISZ {
	return(T_ISZ);
}

AND {
	return(T_AND);
}

DSZ {
	return(T_DSZ);
}

JSM {
	return(T_JSM);
}

JMP {
	return(T_JMP);
}

EX{REG} {
	yylval->opreg = tolower(yytext[2]) - 'a';
	return(T_EXR);
}

DMA {
	return(T_DMA);
}

CM{REG} {
	yylval->opreg = tolower(yytext[2]) - 'a';
	return(T_CMR);
}

TC{REG} {
	yylval->opreg = tolower(yytext[2]) - 'a';
	return(T_TCR);
}

A{REG}R {
	yylval->opreg = tolower(yytext[1]) - 'a';
	return(T_ARR);
}

S{REG}R {
	yylval->opreg = tolower(yytext[1]) - 'a';
	return(T_SRR);
}

S{REG}L {
	yylval->opreg = tolower(yytext[1]) - 'a';
	return(T_SRL);
}

R{REG}R {
	yylval->opreg = tolower(yytext[1]) - 'a';
	return(T_RRR);
}

SZ{REG} {
	yylval->opreg = tolower(yytext[2]) - 'a';
	return(T_SZR);
}

RZ{REG} {
	yylval->opreg = tolower(yytext[2]) - 'a';
	return(T_RZR);
}

SI{REG} {
	yylval->opreg = tolower(yytext[2]) - 'a';
	return(T_SIR);
}

RI{REG} {
	yylval->opreg = tolower(yytext[2]) - 'a';
	return(T_RIR);
}

SL{REG} {
	yylval->opreg = tolower(yytext[2]) - 'a';
	return(T_SLR);
}

S{REG}M {
	yylval->opreg = tolower(yytext[1]) - 'a';
	return(T_SRM);
}

S{REG}P {
	yylval->opreg = tolower(yytext[1]) - 'a';
	return(T_SRP);
}

SES {
	return(T_SES);
}

SEC {
	return(T_SEC);
}

STF {
	return(T_STF);
}

CLF {
	return(T_CLF);
}

SFC {
	return(T_SFC);
}

SLR {
	return(T_SLR);
}


SFS {
	return(T_SFS);
}

CLC {
	return(T_CLC);
}

STC {
	return(T_STC);
}

OT{REG} {
	yylval->opreg = tolower(yytext[2]) - 'a';
	return(T_OTR);
}

LI{REG} {
	yylval->opreg = tolower(yytext[2]) - 'a';
	return(T_LIR);
}

MI{REG} {
	yylval->opreg = tolower(yytext[2]) - 'a';
	return(T_MIR);
}

RET {
	return(T_RET);
}

MOV {
	return(T_MOV);
}

CLR {
	return(T_CLR);
}

XFR {
	return(T_XFR);
}

MRX {
	return(T_MRX);
}

MRY {
	return(T_MRY);
}

MLS {
	return(T_MLS);
}

DRS {
	return(T_DRS);
}

DLS {
	return(T_DLS);
}

FXA {
	return(T_FXA);
}

FMP {
	return(T_FMP);
}

FDV {
	return(T_FDV);
}

CMX {
	return(T_CMX);
}

CMY {
	return(T_CMY);
}

MDI {
	return(T_MDI);
}

NRM {
	return(T_NRM);
}

	/* register argument */

{REG} {
	yylval->argreg = tolower(yytext[0]) - 'a';
	return(REG);
}

	/* pseudo-opcodes */

ORG {
	return(T_ORG);
}


DW {
	return(T_DW);
}

DB {
	return(T_DB);
}

BSS {
	return(T_BSS);
}

EQU {
	return(T_EQU);
}

ASC {
	return(T_ASC);
}

NOP {
	return(T_NOP);
}

	/* instruction modifiers */

,{MOD}+ {
	int i;

	yylval->modifier = 0;

	for (i = 0; i < strlen(yytext); i++) {
		switch(tolower(yytext[i])) {
		case ('i'):
			yylval->modifier |= MOD_INDIRECT;
			break;
		case ('s'):
			yylval->modifier |= MOD_SET;
			break;
		case ('c'):
			yylval->modifier |= MOD_CLEAR;
			break;
		}
	}
			
	return(MODIFIER);
}

	/* 'public' keyword for label scope */

PUBLIC {
	return(PUBLIC);
}

	/* C-style strings */

\"(\\.|[^\\"])*\" {

	/* remove the quotation marks before returning yylval */

	yytext[strlen(yytext) - 1] = '\0';
	yylval->string = strdup(&yytext[1]);

	return(STRING);
}

	/* label */

{ID} 	{
	yylval->string = strdup(yytext);
	return(LABEL);
}

	/* C-style shift operators */

"<<" { 
	return(SHL);
}

">>" {
	return(SHR);
}

	/* C-style single line and multiline comments */

"//"(.*)$ {} 	// single line comment

<INITIAL>{
     "/*"              BEGIN(IN_COMMENT);
}

<IN_COMMENT>{
     "*/"      BEGIN(INITIAL);
     [^*\n]+   // eat comment in chunks
     "*"       // eat the lone star
     \n        { yycolumn = 1; }
}

[ \t]+ 	{} /* eat whitespace */

\\[ \t]*\n  	{ yycolumn = 1; }	/* allow sloppy line continuations */

\n 	{ yycolumn = 1; yylineoffset = yyoffset; return(yytext[0]);}

.	{ return(yytext[0]); }

%%
